
__kesl_options_present ()
{
    local opts=("$@")
    for opt in "${opts[@]}";do
        if [[ "$COMP_LINE " =~ " $opt " ]];then
            return 0;
        fi
    done
    return 1
}

__kesl_add_uniq_options ()
{
    for opt in "$@";do
        if [[ ! "$COMP_LINE " =~ " $opt " ]];then
            opts+=( "$opt" )
        fi
    done
    return 0
}

_kesl_control ()
{
    local cur prev first
    COMPREPLY=()
    cur=${COMP_WORDS[COMP_CWORD]}

    local KESL_A_OPTS=(--get-app-list --get-categories)
    local KESL_B_OPTS=(--mass-remove --restore)
    local KESL_C_OPTS=(--get-container-settings --set-container-settings)
    local KESL_D_OPTS=(--get-device-list)
    local KESL_F_OPTS=(--add-rule --del-rule --move-rule --add-zone --del-zone)
    local KESL_H_OPTS=(--get-blocked-hosts --allow-hosts)
    local KESL_L_OPTS=(--add-active-key --add-reserve-key --remove-active-key --remove-reserve-key --load-mdr-blob --remove-mdr-blob)
    local KESL_N_OPTS=(--clear-web-auto-excluded --get-net-settings --set-net-settings --add-certificate --remove-certificate --list-certificates)
    local KESL_R_OPTS=(--add-kataedr-server-certificate --remove-kataedr-server-certificate --query-kataedr-server-certificate --add-kataedr-client-certificate --remove-kataedr-client-certificate --query-kataedr-client-certificate)
    local KESL_S_OPTS=(--app-info --omsinfo)
    local KESL_T_OPTS=(--create-task --delete-task --export-settings --get-app-settings --get-task-list --get-task-state --get-settings --import-settings --update-application --set-settings --set-app-settings --start-task --stop-task --suspend-task --resume-task --get-schedule --set-schedule --scan-container --scan-file)
    local KESL_U_OPTS=(--get-user-list --grant-role --revoke-role)

    if [[ $COMP_CWORD -le 1 ]];then
        COMPREPLY=($( compgen -W "-A -B -C -D -E -F -H -L -N -R -S -T -U -W ${KESL_A_OPTS[*]} ${KESL_B_OPTS[*]} ${KESL_C_OPTS[*]} ${KESL_D_OPTS[*]} ${KESL_F_OPTS[*]} ${KESL_H_OPTS[*]} ${KESL_L_OPTS[*]} ${KESL_N_OPTS[*]} ${KESL_R_OPTS[*]} ${KESL_S_OPTS[*]} ${KESL_T_OPTS[*]} ${KESL_U_OPTS[*]}" -- $cur ))
        return 0
    fi

    first=${COMP_WORDS[1]}
    prev=${COMP_WORDS[COMP_CWORD-1]}

    if [[ $prev =~ (--(create-task|add-certificate|(remove|query)-kataedr-(server|client)-certificate)) ]]; then
        return 0
    fi

    if [[ $prev = "--set-app-settings" ]]; then
        COMPREPLY=($( compgen -W "$( kesl-control --get-app-settings|sed -e 's/=.*/=/' )" -- $cur ))
        return 0
    fi

    if [[ $prev =~ (--((start|stop|suspend|resume|delete)-task|get-task-state|(get|set)-(settings|schedule))) ]]; then
        COMPREPLY=($( compgen -W "$( kesl-control --get-task-list|grep '^Name: '|sed -e 's/^Name: //' )" -- $cur ))
        return 0
    fi

    if [[ $prev =~ (--(file|add-(active|reserve)-key|load-mdr-blob|scan-file|add-kataedr-(server|client)-certificate)) ]];then
        return 0
    fi

    if [[ $prev =~ (--(add|del)-zone) ]]; then
        COMPREPLY=($( compgen -W "Public Local Trusted" $cur ))
        return 0
    fi

    if [[ $first = "-N" && $prev = "--query" ]]; then
        COMPREPLY=($( compgen -W "auto kl user" $cur ))
        return 0
    fi

    if [[ $prev =~ (--(grant|revoke)-role) ]]; then
        COMPREPLY=($( compgen -W "admin user audit" $cur ))
        return 0
    fi

    if __kesl_options_present --grant-role || __kesl_options_present --revoke-role ; then
        COMPREPLY=($( compgen -u $cur ))
        return 0
    fi

    local -a opts=()

    if [[ $COMP_CWORD -eq 2 ]];then
        case "$first" in
            -A)
                opts+=( "${KESL_A_OPTS[@]}" )
                ;;
            -B)
                opts+=( "${KESL_B_OPTS[@]}" )
                ;;
            -C)
                opts+=( "${KESL_C_OPTS[@]}" )
                ;;
            -D)
                opts+=( "${KESL_D_OPTS[@]}" )
                ;;
            -F)
                opts+=( "${KESL_F_OPTS[@]}" )
                ;;
            -H)
                opts+=( "${KESL_H_OPTS[@]}" )
                ;;
            -L)
                opts+=( "${KESL_L_OPTS[@]}" )
                ;;
            -N)
                opts+=( "${KESL_N_OPTS[@]}" )
                ;;
            -R)
                opts+=( "${KESL_R_OPTS[@]}" )
                ;;
            -S)
                opts+=( "${KESL_S_OPTS[@]}" )
                ;;
            -T)
                opts+=( "${KESL_T_OPTS[@]}" )
                ;;
            -U)
                opts+=( "${KESL_U_OPTS[@]}" )
                ;;
        esac
    fi

    if [[ " -B -E -F -L -N -W " =~ " $first " ]]; then
        if ! __kesl_options_present --query --restore ; then
            opts+=( --query )
        elif [[ " -B -E " =~ " $first " ]]; then
            __kesl_add_uniq_options -n
        fi
    fi

    if [[ "$COMP_LINE " =~ ( --(restore|query|create-task|omsinfo|(export|import|get|set)[a-z-]*settings|(get|set)-schedule|add-certificate) ) ]];then
        __kesl_add_uniq_options --file
    fi

    if [[ $first = "-E" && $prev = "--query" ]]; then
        opts+=( --db )
    fi

    if __kesl_options_present --add-rule ; then
        if [[ $prev = "--action" ]]; then
            COMPREPLY=($(compgen -W "allow block" $cur))
            return 0
        fi
        if [[ $prev = "--protocol" ]]; then
            COMPREPLY=($(compgen -W "any tcp udp icmp icmpv6 igmp gre" $cur))
            return 0
        fi
        if [[ $prev = "--direction" ]]; then
            COMPREPLY=($(compgen -W "in out inout inpacket outpacket inoutpacket" $cur))
            return 0
        fi
        if [[ $prev = "--remote" ]]; then
            COMPREPLY=($(compgen -W "any trusted local public" $cur))
            return 0
        fi
        if [[ $prev = "--local" ]]; then
            COMPREPLY=($(compgen -W "any" $cur))
            return 0
        fi
        if [[ $prev =~ (--(name|at)) ]]; then
            return 0
        fi
        __kesl_add_uniq_options --name --action --protocol --direction --remote --local --at
    fi

    if __kesl_options_present --del-rule --move-rule ; then
        if __kesl_options_present --index ; then
            return 0
        fi
        if [[ $prev = "--name" ]]; then
            local IFS=$'\n'
            local WORD_LIST=($( kesl-control --get-settings Firewall_Management|grep '^Name='|sed -e 's/^Name=//' ))
            local CANDIDATES=($(compgen -W "${WORD_LIST[*]}" -- "$cur"))
            if [[ ${#CANDIDATES[*]} -eq 0 ]]; then
                COMPREPLY=()
            else
                COMPREPLY=($(printf '%q\n' "${CANDIDATES[@]}"))
            fi
            return 0
        fi
        if [[ $prev = "--at" ]]; then
            return 0
        fi
        __kesl_add_uniq_options --name --index
        __kesl_options_present --move-rule && __kesl_add_uniq_options --at
    fi

    if __kesl_options_present --add-zone --del-zone ; then
        if __kesl_options_present --address --index ; then
            return 0
        fi
        __kesl_add_uniq_options --address
        __kesl_options_present --del-zone && __kesl_add_uniq_options --index
    fi

    if [[ $prev = "--allow-hosts" ]]; then
        COMPREPLY=($( compgen -W "$( kesl-control --get-blocked-hosts|grep -E 'Host *:'|sed -e 's/.*Host *: *//' )" -- $cur ))
        return 0
    fi

    if __kesl_options_present --create-task ; then
        if [[ $prev = "--type" ]]; then
            COMPREPLY=($(compgen -W "ODS Update Rollback" $cur))
            return 0
        fi
        __kesl_add_uniq_options --type --file
    fi

    if __kesl_options_present --set-settings ; then
        opts+=( --add-path --del-path --add-exclusion --del-exclusion )
        __kesl_add_uniq_options --set-to-default
    fi

    if __kesl_options_present --start-task ; then
        __kesl_add_uniq_options -W --progress
    fi

    if __kesl_options_present --stop-task ; then
        __kesl_add_uniq_options -W
    fi

    if __kesl_options_present --scan-file ; then
        if [[ $prev = "--action" ]]; then
            COMPREPLY=($(compgen -W "Disinfect Remove Recommended Skip Block" $cur))
            return 0
        fi
        __kesl_add_uniq_options --action
    fi

    if [[ $COMP_LINE =~ (--(get-((app|device|task)-(list|state)|categories)|query|app-info|(get|set|export|import)-((app|container|net)-)?settings|get-schedule)($| )) ]];then
        __kesl_add_uniq_options --json
    fi

    COMPREPLY=($(compgen -W "${opts[*]}" -- $cur))

    return 0
}

complete -F _kesl_control -o default kesl-control

